@@ -1,5 +1,4 @@
-use std::fmt::Debug;
-
+#[cfg(feature = "audio")]
 pub mod music_player;
 
 #[derive(Debug, Clone, Copy, PartialEq)]
