@@ -1,9 +1,4 @@
-use super::{
-    action::{Action, ActionOutput, ActionSituation, Advantage, EngineAction},
-    constants::*,
-    game::Game,
-    types::*,
-};
+use super::{action::*, constants::*, game::Game, types::*};
 use crate::world::{
     constants::{MoraleModifier, TirednessCost},
     skill::GameSkill,
@@ -12,53 +7,59 @@ use rand::{seq::IndexedRandom, Rng};
 use rand_chacha::ChaCha8Rng;
 use std::collections::HashMap;
 
-#[derive(Debug, Default)]
-pub struct OffTheScreen;
+pub(crate) fn execute(
+    input: &ActionOutput,
+    game: &Game,
+    action_rng: &mut ChaCha8Rng,
+    description_rng: &mut ChaCha8Rng,
+) -> Option<ActionOutput> {
+    let attacking_players = game.attacking_players();
+    let defending_players = game.defending_players();
 
-impl EngineAction for OffTheScreen {
-    fn execute(input: &ActionOutput, game: &Game, action_rng: &mut ChaCha8Rng, description_rng: &mut ChaCha8Rng) -> Option<ActionOutput> {
-        let attacking_players = game.attacking_players();
-        let defending_players = game.defending_players();
+    let play_idx = match input.attackers.len() {
+        0 => sample_player_index(action_rng, [6, 1, 2, 0, 0])?,
+        _ => input.attackers[0],
+    };
+    let target_idx = match input.attackers.len() {
+        2 => input.attackers[1],
+        _ => {
+            let mut weights = [1, 2, 3, 3, 2];
+            weights[play_idx] = 0;
+            sample_player_index(action_rng, weights)?
+        }
+    };
 
-        let play_idx = match input.attackers.len() {
-            0 => Self::sample(action_rng, [6, 1, 2, 0, 0])?,
-            _ => input.attackers[0],
-        };
-        let target_idx = match input.attackers.len() {
-            2 => input.attackers[1],
-            _ => {
-                let mut weights = [1, 2, 3, 3, 2];
-                weights[play_idx] = 0;
-                Self::sample(action_rng, weights)?
-            }
-        };
+    let playmaker = attacking_players[play_idx];
+    let playmaker_defender = defending_players[play_idx];
 
-        let playmaker = attacking_players[play_idx];
-        let playmaker_defender = defending_players[play_idx];
+    let target = attacking_players[target_idx];
+    let target_defender = defending_players[target_idx];
 
-        let target = attacking_players[target_idx];
-        let target_defender = defending_players[target_idx];
+    let mut attack_stats_update: GameStatsMap = HashMap::new();
+    let mut playmaker_update = GameStats {
+        extra_tiredness: TirednessCost::MEDIUM,
+        ..Default::default()
+    };
 
-        let mut attack_stats_update: GameStatsMap = HashMap::new();
-        let mut playmaker_update = GameStats::default();
-        playmaker_update.extra_tiredness = TirednessCost::MEDIUM;
+    let mut defense_stats_update: GameStatsMap = HashMap::new();
+    let mut target_defender_update = GameStats {
+        extra_tiredness: TirednessCost::MEDIUM,
+        ..Default::default()
+    };
 
-        let mut defense_stats_update: GameStatsMap = HashMap::new();
-        let mut target_defender_update = GameStats::default();
-        target_defender_update.extra_tiredness = TirednessCost::MEDIUM;
+    let timer_increase = 4 + action_rng.random_range(0..=2);
 
-        let timer_increase = 4 + action_rng.random_range(0..=2);
+    let atk_result = playmaker.roll(action_rng)
+        + playmaker.mental.vision.game_value()
+        + playmaker.technical.passing.game_value()
+        + target.mental.intuition.game_value();
 
-        let atk_result = playmaker.roll(action_rng)
-            + playmaker.mental.vision.game_value()
-            + playmaker.technical.passing.game_value()
-            + target.mental.intuition.game_value();
+    let def_result = playmaker_defender.roll(action_rng)
+        + target_defender.defense.perimeter_defense.game_value()
+        + target_defender.athletics.quickness.game_value();
 
-        let def_result = playmaker_defender.roll(action_rng)
-            + target_defender.defense.perimeter_defense.game_value()
-            + target_defender.athletics.quickness.game_value();
-
-        let mut result = match atk_result as i16 - def_result as i16 + Self::tactic_modifier(game, &Action::OffTheScreen) {
+    let mut result = match atk_result  - def_result + Action::OffTheScreen.tactic_modifier(game.attacking_team().tactic,
+                game.defending_team().tactic) {
             x if x >= ADV_ATTACK_LIMIT => ActionOutput {
                 possession: input.possession,
                 advantage: Advantage::Attack,
@@ -169,7 +170,7 @@ impl EngineAction for OffTheScreen {
                 playmaker_update.extra_morale += MoraleModifier::SMALL_MALUS;
                 target_defender_update.extra_morale += MoraleModifier::MEDIUM_BONUS;
 
-                let with_steal =  def_result >= 3* NUMBER_OF_ROLLS;
+                let with_steal =  def_result >= 3 * NUMBER_OF_ROLLS as i16;
 
                 if with_steal{
                 target_defender_update.steals = 1;
@@ -196,7 +197,7 @@ impl EngineAction for OffTheScreen {
                         [format!(
                             "{} passes to {} off the screen, but the pass is too high and goes out of bounds.",
                             playmaker.info.short_name(), target.info.short_name()
-                        ), 
+                        ),
                         format!(
                             "{} sends it to {}, but the pass is off the mark.",
                             playmaker.info.short_name(), target.info.short_name()
@@ -216,10 +217,9 @@ impl EngineAction for OffTheScreen {
             }
         };
 
-        attack_stats_update.insert(playmaker.id, playmaker_update);
-        defense_stats_update.insert(target_defender.id, target_defender_update);
-        result.attack_stats_update = Some(attack_stats_update);
-        result.defense_stats_update = Some(defense_stats_update);
-        Some(result)
-    }
+    attack_stats_update.insert(playmaker.id, playmaker_update);
+    defense_stats_update.insert(target_defender.id, target_defender_update);
+    result.attack_stats_update = Some(attack_stats_update);
+    result.defense_stats_update = Some(defense_stats_update);
+    Some(result)
 }
