@@ -2,10 +2,12 @@ mod asteroid;
 mod collector;
 mod collisions;
 mod constants;
+mod entity;
 mod fragment;
 mod networking;
 mod particle;
 mod projectile;
+mod shield;
 mod space;
 mod space_callback;
 mod spaceship;
