@@ -1,73 +1,108 @@
 use super::collisions::HitBox;
-use super::networking::ImageType;
 use super::space_callback::SpaceCallback;
 use super::utils::{body_data_from_image, EntityState};
 use super::{constants::*, traits::*};
-use crate::image::color_map::ColorMap;
 use crate::image::components::{ImageComponent, SizedImageComponent};
 use crate::image::spaceship::SpaceshipImage;
 use crate::image::utils::open_image;
-use crate::register_impl;
+use crate::space_adventure::entity::Entity;
 use crate::space_adventure::visual_effects::VisualEffect;
 use crate::space_adventure::Direction;
 use crate::types::*;
 use crate::world::constants::{FUEL_CONSUMPTION_PER_UNIT_STORAGE, SPEED_PENALTY_PER_UNIT_STORAGE};
 use crate::world::resources::Resource;
-use crate::world::spaceship::SpaceshipPrefab;
 use crate::{image::types::Gif, types::AppResult, world::spaceship::Spaceship};
 use glam::{I16Vec2, Vec2};
 use image::imageops::{rotate270, rotate90};
 use image::{Pixel, Rgba, RgbaImage};
-use itertools::Itertools;
 use rand::seq::IndexedRandom;
 use rand::{Rng, SeedableRng};
 use rand_chacha::ChaCha8Rng;
 use std::collections::HashMap;
-use strum::IntoEnumIterator;
 
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub enum ShooterState {
-    Ready { charge: f32 },
-    Shooting { charge: f32, recoil: f32 },
-    Recharging { charge: f32 },
+    Ready,
+    Shooting { recoil: f32 },
 }
 
 impl ShooterState {
-    const SHOOTING_CHARGE_COST: f32 = 1.0;
-    const MAX_CHARGE: f32 = 100.0;
-    const CHARGE_RECOVERY_SPEED: f32 = 5.0;
-    const RECHARGE_RECOVERY_SPEED: f32 = 3.0;
+    const SHOOTING_CHARGE_COST: f32 = 2.05;
 }
 
 #[derive(Debug)]
-struct Shooter {
+struct ShooterInSpaceAdventure {
     pub positions: Vec<I16Vec2>,
     pub damage: f32,
     pub max_recoil: f32,
+    pub autofire: bool,
     pub state: ShooterState,
 }
 
-impl Shooter {
+impl ShooterInSpaceAdventure {
     pub fn new(positions: Vec<I16Vec2>, damage: f32, fire_rate: f32) -> Self {
         Self {
             positions,
             damage,
             max_recoil: 1.0 / fire_rate,
-            state: ShooterState::Ready {
-                charge: ShooterState::MAX_CHARGE,
-            },
+            autofire: false,
+            state: ShooterState::Ready,
         }
     }
 
-    pub fn set_state(&mut self, state: ShooterState) {
+    fn set_state(&mut self, state: ShooterState) {
+        self.state = state;
+    }
+}
+
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub enum ChargeUnitState {
+    Ready,
+    Recharging,
+}
+
+#[derive(Debug)]
+struct ChargeUnitInSpaceAdventure {
+    pub current_charge: f32,
+    pub max_charge: f32,
+    pub state: ChargeUnitState,
+}
+
+impl ChargeUnitInSpaceAdventure {
+    const CHARGE_RECOVERY_PER_SECOND: f32 = 2.5;
+    const RECHARGE_RECOVERY_PER_SECOND: f32 = 2.25;
+
+    pub fn new(max_charge: f32) -> Self {
+        Self {
+            current_charge: max_charge,
+            max_charge,
+            state: ChargeUnitState::Ready,
+        }
+    }
+
+    fn set_state(&mut self, state: ChargeUnitState) {
         self.state = state;
     }
 
-    pub fn shoot(&mut self, charge: f32) {
-        self.set_state(ShooterState::Shooting {
-            charge,
-            recoil: self.max_recoil,
-        })
+    fn sub_charge(&mut self, value: f32) {
+        self.current_charge = (self.current_charge - value).clamp(0.0, self.max_charge)
+    }
+
+    fn add_charge(&mut self, value: f32) {
+        self.current_charge = (self.current_charge + value).clamp(0.0, self.max_charge)
+    }
+
+    fn recharge(&mut self, deltatime: f32) {
+        match self.state {
+            ChargeUnitState::Ready => {
+                self.add_charge(ChargeUnitInSpaceAdventure::CHARGE_RECOVERY_PER_SECOND * deltatime);
+            }
+            ChargeUnitState::Recharging => {
+                self.add_charge(
+                    ChargeUnitInSpaceAdventure::RECHARGE_RECOVERY_PER_SECOND * deltatime,
+                );
+            }
+        }
     }
 }
 
@@ -75,7 +110,7 @@ impl Shooter {
 pub struct SpaceshipEntity {
     id: usize,
     is_player: bool,
-    base_spaceship: Spaceship,
+    spaceship: Spaceship,
     resources: ResourceMap,
     used_storage_capacity: u32, // Not necessary, we keep it to avoid recalculating them every time.
     storage_capacity: u32,
@@ -88,7 +123,8 @@ pub struct SpaceshipEntity {
     // Maps with hit box points, Point -> is_border in local coordinates.
     hit_boxes: Vec<HitBox>,
     current_durability: f32,
-    durability: f32,
+    max_durability: f32,
+    damage_reduction: f32,
     base_thrust: f32,
     base_speed: f32,
     maneuverability: f32,
@@ -98,11 +134,13 @@ pub struct SpaceshipEntity {
     tick: usize,
     gif: Gif,
     engine_exhaust: Vec<I16Vec2>, // Position of exhaust in relative coords
-    shooter: Shooter,
-    auto_shoot: bool,
-    collector_id: usize,
+    shooter: Option<ShooterInSpaceAdventure>, // FIXME: it should be its own entity (as the engine and storage) so they can be damaged separately.
+    charge_unit: ChargeUnitInSpaceAdventure,
+    collector_id: Option<usize>,
+    shield_id: Option<usize>,
     visual_effects: VisualEffectMap,
     releasing_scraps: bool,
+    pending_input_callbacks: Vec<SpaceCallback>,
 }
 
 impl Body for SpaceshipEntity {
@@ -126,7 +164,7 @@ impl Body for SpaceshipEntity {
         let rng = &mut ChaCha8Rng::from_os_rng();
 
         // Generate fire particle if ship is damaged
-        for _ in self.current_durability as usize..(0.5 * self.durability) as usize {
+        for _ in self.current_durability as usize..(0.5 * self.max_durability) as usize {
             if rng.random_bool(0.2) {
                 let position = self.center().as_vec2()
                     + Vec2::new(rng.random_range(-1.0..1.0), rng.random_range(-1.0..1.0));
@@ -198,30 +236,41 @@ impl Body for SpaceshipEntity {
         self.position += self.velocity * deltatime;
         self.acceleration = Vec2::ZERO;
 
-        // The spaceship must always remain on screen
-        let min_position = (MAX_ENTITY_POSITION - SCREEN_SIZE).as_vec2() / 2.0;
-        let max_position = min_position + SCREEN_SIZE.as_vec2() - self.size().as_vec2();
+        if self.is_player() {
+            // The spaceship must always remain on screen
+            let min_position = (MAX_ENTITY_POSITION - SCREEN_SIZE).as_vec2() / 2.0;
+            let max_position = min_position + SCREEN_SIZE.as_vec2() - self.size().as_vec2();
 
-        if self.position.x < min_position.x {
-            self.position.x = min_position.x;
-            self.velocity.x = 0.0;
-        } else if self.position.x > max_position.x {
-            self.position.x = max_position.x;
-            self.velocity.x = 0.0;
+            if self.position.x < min_position.x {
+                self.position.x = min_position.x;
+                self.velocity.x = 0.0;
+            } else if self.position.x > max_position.x {
+                self.position.x = max_position.x;
+                self.velocity.x = 0.0;
+            }
+
+            if self.position.y < min_position.y {
+                self.position.y = min_position.y;
+                self.velocity.y = 0.0;
+            } else if self.position.y > max_position.y {
+                self.position.y = max_position.y;
+                self.velocity.y = 0.0;
+            }
         }
 
-        if self.position.y < min_position.y {
-            self.position.y = min_position.y;
-            self.velocity.y = 0.0;
-        } else if self.position.y > max_position.y {
-            self.position.y = max_position.y;
-            self.velocity.y = 0.0;
+        if let Some(id) = self.collector_id {
+            callbacks.push(SpaceCallback::SetCenterPosition {
+                id,
+                center: self.center(),
+            });
         }
 
-        callbacks.push(SpaceCallback::SetPosition {
-            id: self.collector_id,
-            position: self.center(),
-        });
+        if let Some(id) = self.shield_id {
+            callbacks.push(SpaceCallback::SetCenterPosition {
+                id,
+                center: self.center(),
+            });
+        }
 
         callbacks
     }
@@ -232,25 +281,14 @@ impl Sprite for SpaceshipEntity {
         &self.gif[self.frame()]
     }
 
-    fn network_image_type(&self) -> ImageType {
-        ImageType::Spaceship {
-            hull: self.base_spaceship.hull,
-            engine: self.base_spaceship.engine,
-            storage: self.base_spaceship.storage,
-            color_map: self.base_spaceship.image.color_map,
-        }
-    }
-
     fn should_apply_visual_effects<'a>(&self) -> bool {
         !self.visual_effects.is_empty()
     }
 
     fn apply_visual_effects<'a>(&'a self, image: &'a RgbaImage) -> RgbaImage {
         let mut image = image.clone();
-        if !self.visual_effects.is_empty() {
-            for (effect, time) in self.visual_effects.iter() {
-                effect.apply(self, &mut image, *time);
-            }
+        for (effect, time) in self.visual_effects.iter() {
+            effect.apply(self, &mut image, *time);
         }
         image
     }
@@ -274,7 +312,7 @@ impl Sprite for SpaceshipEntity {
     }
 }
 
-impl Entity for SpaceshipEntity {
+impl GameEntity for SpaceshipEntity {
     fn set_id(&mut self, id: usize) {
         self.id = id;
     }
@@ -288,97 +326,106 @@ impl Entity for SpaceshipEntity {
     }
 
     fn update(&mut self, deltatime: f32) -> Vec<SpaceCallback> {
+        let mut callbacks = self.pending_input_callbacks.to_owned();
+        self.pending_input_callbacks.clear();
+
         // This is only triggered for enemy ships and not for the player ship.
-        if !self.is_player && self.current_durability() == 0 {
-            return vec![SpaceCallback::DestroyEntity { id: self.id }];
-        }
+        if !self.is_player() {
+            if self.current_durability() == 0 {
+                callbacks.push(SpaceCallback::DestroyEntity { id: self.id });
 
-        if !self.is_player {
-            let rng = &mut rand::rng();
-            if let ShooterState::Ready { charge } = self.shooter.state {
-                if !self.auto_shoot
-                    && charge > ShooterState::MAX_CHARGE * rng.random_range(0.25..1.0)
-                {
-                    self.auto_shoot = true;
+                if let Some(id) = self.collector_id {
+                    callbacks.push(SpaceCallback::DestroyEntity { id });
                 }
-                if charge <= 0.25 {
-                    self.auto_shoot = false;
+
+                if let Some(id) = self.shield_id {
+                    callbacks.push(SpaceCallback::DestroyEntity { id });
                 }
+
+                return callbacks;
             }
+
+            callbacks.push(SpaceCallback::TrackPlayer { id: self.id });
         }
 
-        let mut callbacks = vec![];
         callbacks.append(&mut self.update_body(deltatime));
         callbacks.append(&mut self.update_sprite(deltatime));
 
-        match self.shooter.state {
-            ShooterState::Shooting { charge, recoil } => {
-                if recoil == self.shooter.max_recoil {
-                    for shooter_position in self.shooter.positions.iter() {
-                        callbacks.push(SpaceCallback::GenerateProjectile {
-                            shot_by_id: self.id(),
-                            position: self.position + shooter_position.as_vec2(),
-                            velocity: Vec2::X * 100.0 * self.orientation() as f32,
-                            color: Rgba([
-                                25,
-                                125,
-                                55 + (200.0 * charge / ShooterState::MAX_CHARGE) as u8,
-                                255,
-                            ]),
-                            damage: self.shooter.damage,
-                        });
-                    }
-                    let new_charge = charge - ShooterState::SHOOTING_CHARGE_COST;
-                    if new_charge > 0.0 {
-                        let new_recoil = recoil - deltatime;
-                        self.shooter.set_state(ShooterState::Shooting {
-                            charge: new_charge,
-                            recoil: new_recoil,
-                        });
-                    } else {
-                        self.shooter
-                            .set_state(ShooterState::Recharging { charge: 0.0 });
-                    }
-                } else {
-                    let new_recoil = recoil - deltatime;
-                    if new_recoil > 0.0 {
-                        self.shooter.set_state(ShooterState::Shooting {
-                            charge,
-                            recoil: new_recoil,
-                        });
-                    } else {
-                        self.shooter.set_state(ShooterState::Ready { charge });
-                    }
-                }
+        if self.current_charge() == 0 {
+            self.charge_unit.set_state(ChargeUnitState::Recharging);
+
+            if let Some(id) = self.collector_id {
+                callbacks.push(SpaceCallback::DeactivateEntity { id });
             }
 
-            ShooterState::Ready { charge } => {
-                if self.auto_shoot {
-                    self.shooter.set_state(ShooterState::Shooting {
-                        charge,
-                        recoil: self.shooter.max_recoil,
-                    });
-                } else if charge < ShooterState::MAX_CHARGE {
-                    let new_charge = (charge + ShooterState::CHARGE_RECOVERY_SPEED * deltatime)
-                        .min(ShooterState::MAX_CHARGE);
-                    self.shooter
-                        .set_state(ShooterState::Ready { charge: new_charge });
-                }
+            if let Some(id) = self.shield_id {
+                callbacks.push(SpaceCallback::DeactivateEntity { id });
+            }
+        }
+
+        if self.charge_unit.state == ChargeUnitState::Ready {
+            // Turn on collector again automatically.
+            // FIXME: this is unnecessary most of the time, as it is already active.
+            if let Some(id) = self.collector_id {
+                callbacks.push(SpaceCallback::ActivateEntity { id });
             }
 
-            ShooterState::Recharging { charge } => {
-                let new_charge = charge + ShooterState::RECHARGE_RECOVERY_SPEED * deltatime;
-                if new_charge < ShooterState::MAX_CHARGE {
-                    self.shooter
-                        .set_state(ShooterState::Recharging { charge: new_charge });
-                } else {
-                    self.shooter.set_state(ShooterState::Ready {
-                        charge: ShooterState::MAX_CHARGE,
-                    });
+            if let Some(shooter) = self.shooter.as_mut() {
+                match shooter.state {
+                    ShooterState::Ready => {
+                        if shooter.autofire {
+                            shooter.set_state(ShooterState::Shooting {
+                                recoil: shooter.max_recoil,
+                            });
+                        }
+                    }
+
+                    ShooterState::Shooting { recoil } => {
+                        // If recoil is at max, actually shoot
+                        if recoil == shooter.max_recoil {
+                            for shooter_position in shooter.positions.iter() {
+                                callbacks.push(SpaceCallback::GenerateProjectile {
+                                    shot_by_id: self.id,
+                                    position: self.position + shooter_position.as_vec2(),
+                                    velocity: Vec2::X
+                                        * 100.0
+                                        * if self.is_player { 1.0 } else { -1.0 },
+                                    color: Rgba([
+                                        25,
+                                        125,
+                                        (55.0
+                                            + (200.0 * self.charge_unit.current_charge
+                                                / self.charge_unit.max_charge))
+                                            .min(255.0)
+                                            as u8,
+                                        255,
+                                    ]),
+                                    damage: shooter.damage,
+                                });
+                            }
+                            self.charge_unit
+                                .sub_charge(ShooterState::SHOOTING_CHARGE_COST);
+
+                            shooter.set_state(ShooterState::Shooting {
+                                recoil: shooter.max_recoil - deltatime, // Reduce it a bit so that it does not shoot again immediately.
+                            })
+                        }
+                        // Otherwise reduce recoil
+                        else {
+                            let new_recoil = recoil - deltatime;
+                            if new_recoil > 0.0 {
+                                shooter.set_state(ShooterState::Shooting { recoil: new_recoil });
+                            } else {
+                                shooter.set_state(ShooterState::Ready);
+                            }
+                        }
+                    }
                 }
             }
         }
 
+        self.charge_unit.recharge(deltatime);
+
         if self.releasing_scraps {
             let rng = &mut rand::rng();
             callbacks.push(SpaceCallback::GenerateParticle {
@@ -395,28 +442,36 @@ impl Entity for SpaceshipEntity {
             });
             self.releasing_scraps = false;
         }
+
         callbacks
     }
 
     fn handle_space_callback(&mut self, callback: SpaceCallback) -> Vec<SpaceCallback> {
         match callback {
+            SpaceCallback::SetAcceleration { acceleration, .. } => {
+                self.thrust_towards(acceleration);
+            }
             SpaceCallback::DestroyEntity { .. } => {
                 let rng = &mut rand::rng();
-                let mut callbacks = vec![SpaceCallback::DestroyEntity {
-                    id: self.collector_id,
-                }];
+                let mut callbacks = vec![];
 
-                let color_map = self.base_spaceship.image.color_map;
+                if let Some(id) = self.collector_id {
+                    callbacks.push(SpaceCallback::DestroyEntity { id });
+                }
+
+                let color_map = self.spaceship.image.color_map;
                 let colors = [color_map.red, color_map.green, color_map.blue];
-                for _ in 0..24 {
+
+                let position = self.center().as_vec2();
+                for _ in 0..32 {
                     let color = colors.choose(rng).expect("There should be one color");
                     callbacks.push(SpaceCallback::GenerateParticle {
-                        position: self.center().as_vec2(),
+                        position,
                         velocity: self.velocity
                             + Vec2::new(
                                 rng.random_range(-10.0..10.0),
                                 rng.random_range(-10.0..10.0),
-                            ),
+                            ) * 3.0,
                         color: color.to_rgba(),
                         particle_state: EntityState::Decaying {
                             lifetime: 5.0 + rng.random_range(-1.5..1.5),
@@ -424,6 +479,30 @@ impl Entity for SpaceshipEntity {
                         layer: rng.random_range(0..=2),
                     });
                 }
+
+                for _ in 4..8 {
+                    callbacks.push(SpaceCallback::GenerateFragment {
+                        position,
+                        velocity: Vec2::new(
+                            rng.random_range(-3.5..3.5),
+                            rng.random_range(-3.5..3.5),
+                        ),
+                        resource: Resource::GOLD,
+                        amount: 1,
+                    });
+                }
+                for _ in 10..16 {
+                    callbacks.push(SpaceCallback::GenerateFragment {
+                        position,
+                        velocity: Vec2::new(
+                            rng.random_range(-3.5..3.5),
+                            rng.random_range(-3.5..3.5),
+                        ) * 2.0,
+                        resource: Resource::SCRAPS,
+                        amount: 2,
+                    });
+                }
+
                 return callbacks;
             }
 
@@ -447,6 +526,12 @@ impl Entity for SpaceshipEntity {
                 self.used_storage_capacity = self.resources.used_storage_capacity()
             }
 
+            SpaceCallback::ReleaseScraps { .. } => self.release_scraps(),
+            SpaceCallback::Shoot { .. } => self.shoot(),
+            SpaceCallback::ToggleAutofire { .. } => self.toggle_autofire(),
+            SpaceCallback::UseCharge { amount, .. } => {
+                self.charge_unit.sub_charge(amount);
+            }
             _ => {}
         }
         vec![]
@@ -463,9 +548,6 @@ impl Collider for SpaceshipEntity {
     }
 }
 
-register_impl!(ControllableSpaceship for SpaceshipEntity);
-register_impl!(!ResourceFragment for SpaceshipEntity);
-
 impl ControllableSpaceship for SpaceshipEntity {
     fn is_player(&self) -> bool {
         self.is_player
@@ -483,6 +565,10 @@ impl ControllableSpaceship for SpaceshipEntity {
         &self.resources
     }
 
+    fn resources_mut(&mut self) -> &mut ResourceMap {
+        &mut self.resources
+    }
+
     fn storage_capacity(&self) -> u32 {
         self.storage_capacity
     }
@@ -491,21 +577,16 @@ impl ControllableSpaceship for SpaceshipEntity {
         self.max_speed().round() as u32
     }
 
-    fn charge(&self) -> u32 {
-        match self.shooter.state {
-            ShooterState::Ready { charge } => charge,
-            ShooterState::Recharging { charge } => charge,
-            ShooterState::Shooting { charge, .. } => charge,
-        }
-        .round() as u32
+    fn is_recharging(&self) -> bool {
+        self.charge_unit.state == ChargeUnitState::Recharging
     }
 
-    fn shooter_state(&self) -> ShooterState {
-        self.shooter.state
+    fn current_charge(&self) -> u32 {
+        self.charge_unit.current_charge as u32
     }
 
     fn max_charge(&self) -> u32 {
-        ShooterState::MAX_CHARGE.round() as u32
+        self.charge_unit.max_charge as u32
     }
 
     fn thrust(&self) -> u32 {
@@ -520,29 +601,59 @@ impl ControllableSpaceship for SpaceshipEntity {
         self.current_durability.round() as u32
     }
 
-    fn durability(&self) -> u32 {
-        self.durability.round() as u32
+    fn max_durability(&self) -> u32 {
+        self.max_durability.round() as u32
     }
 
     fn handle_player_input(&mut self, input: PlayerInput) {
-        match input {
-            PlayerInput::MoveDown => self.accelerate(Direction::DOWN),
-            PlayerInput::MoveUp => self.accelerate(Direction::UP),
-            PlayerInput::MoveLeft => self.accelerate(Direction::LEFT),
-            PlayerInput::MoveRight => self.accelerate(Direction::RIGHT),
-            PlayerInput::Shoot => self.shoot(),
-            PlayerInput::ToggleAutofire => self.auto_shoot = !self.auto_shoot,
-            PlayerInput::ReleaseScraps => self.release_scraps(),
+        if let Some(cb) = match input {
+            PlayerInput::MoveDown => Some(SpaceCallback::SetAcceleration {
+                id: self.id(),
+                acceleration: Direction::Down.as_vec2(),
+            }),
+            PlayerInput::MoveUp => Some(SpaceCallback::SetAcceleration {
+                id: self.id(),
+                acceleration: Direction::Up.as_vec2(),
+            }),
+            PlayerInput::MoveLeft => Some(SpaceCallback::SetAcceleration {
+                id: self.id(),
+                acceleration: Direction::Left.as_vec2(),
+            }),
+            PlayerInput::MoveRight => Some(SpaceCallback::SetAcceleration {
+                id: self.id(),
+                acceleration: Direction::Right.as_vec2(),
+            }),
+            PlayerInput::Shoot => Some(SpaceCallback::Shoot { id: self.id() }),
+            PlayerInput::ReleaseScraps => Some(SpaceCallback::ReleaseScraps { id: self.id() }),
+            PlayerInput::ToggleAutofire => Some(SpaceCallback::ToggleAutofire { id: self.id() }),
+            PlayerInput::ToggleShield => {
+                self.shield_id.map(|id| SpaceCallback::ToggleShield { id })
+            }
+        } {
+            self.pending_input_callbacks.push(cb);
         }
     }
 }
 
 impl SpaceshipEntity {
-    fn orientation(&self) -> i8 {
-        if self.is_player {
-            return 1;
+    pub fn spaceship(&self) -> &Spaceship {
+        &self.spaceship
+    }
+
+    pub fn shield_id(&self) -> Option<usize> {
+        self.shield_id
+    }
+
+    pub fn toggle_autofire(&mut self) {
+        if let Some(shooter) = self.shooter.as_mut() {
+            shooter.autofire = !shooter.autofire
+        }
+    }
+
+    pub fn set_autofire(&mut self, value: bool) {
+        if let Some(shooter) = self.shooter.as_mut() {
+            shooter.autofire = value;
         }
-        -1
     }
 
     fn thrust(&self) -> f32 {
@@ -559,12 +670,12 @@ impl SpaceshipEntity {
             / (1.0 + FUEL_CONSUMPTION_PER_UNIT_STORAGE * self.used_storage_capacity as f32)
     }
 
-    fn accelerate(&mut self, acceleration: Vec2) {
+    pub fn thrust_towards(&mut self, direction: Vec2) {
         if self.fuel == 0.0 {
             return;
         }
 
-        self.acceleration = acceleration * self.thrust();
+        self.acceleration = direction * self.thrust();
         self.fuel = (self.fuel - self.fuel_consumption()).max(0.0);
         // Keep fuel in resources updated
         self.resources.insert(Resource::FUEL, self.fuel());
@@ -578,12 +689,19 @@ impl SpaceshipEntity {
         self.acceleration.as_i16vec2()
     }
 
-    pub fn add_damage(&mut self, damage: f32) {
-        self.current_durability = (self.current_durability - damage).max(0.0);
+    fn add_damage(&mut self, damage: f32) {
+        self.current_durability =
+            (self.current_durability - damage * self.damage_reduction).max(0.0);
     }
 
     fn shoot(&mut self) {
-        if let ShooterState::Ready { charge } = self.shooter.state { self.shooter.shoot(charge) }
+        if let Some(shooter) = self.shooter.as_mut() {
+            if shooter.state == ShooterState::Ready {
+                shooter.set_state(ShooterState::Shooting {
+                    recoil: shooter.max_recoil,
+                });
+            }
+        }
     }
 
     fn release_scraps(&mut self) {
@@ -592,34 +710,48 @@ impl SpaceshipEntity {
         }
     }
 
-    pub fn from_spaceship(
+    fn from_spaceship(
         spaceship: &Spaceship,
         resources: ResourceMap,
         speed_bonus: f32,
         weapons_bonus: f32,
         fuel: u32,
-        collector_id: usize,
-    ) -> AppResult<Self> {
+        collector_id: Option<usize>,
+        shield_id: Option<usize>,
+        is_player: bool,
+    ) -> AppResult<Entity> {
         let mut gif = vec![];
         let mut hit_boxes = vec![];
         let base_gif = spaceship.compose_image()?;
-        for idx in 0..base_gif.len() {
-            let base_image = rotate90(&base_gif[idx]);
+        for gif_frame in base_gif.iter() {
+            let base_image = if is_player {
+                rotate90(gif_frame)
+            } else {
+                rotate270(gif_frame)
+            };
             let (image, hit_box) = body_data_from_image(&base_image, true);
 
             gif.push(image);
             hit_boxes.push(hit_box);
         }
 
-        let position = Vec2::new(
-            (MAX_ENTITY_POSITION.x - SCREEN_SIZE.x) as f32 / 2.0,
-            0.5 * SCREEN_SIZE.y as f32,
-        );
+        let position = if is_player {
+            Vec2::new(
+                (MAX_ENTITY_POSITION.x as f32 - SCREEN_SIZE.x as f32) / 2.0,
+                0.5 * SCREEN_SIZE.y as f32,
+            )
+        } else {
+            Vec2::new(MAX_ENTITY_POSITION.x as f32, SCREEN_SIZE.y as f32 / 2.0)
+        };
 
         let mut engine_img = open_image(&spaceship.engine.select_mask_file())?;
-        engine_img = rotate90(&engine_img);
+        engine_img = if is_player {
+            rotate90(&engine_img)
+        } else {
+            rotate270(&engine_img)
+        };
 
-        let y_offset = (engine_img.height() - gif[0].height()) / 2;
+        let y_offset = (engine_img.height().saturating_sub(gif[0].height())) / 2;
         let mut engine_exhaust = vec![];
         for x in 0..engine_img.width() {
             for y in 0..engine_img.height() {
@@ -634,7 +766,11 @@ impl SpaceshipEntity {
 
         let size = SpaceshipImage::size(&spaceship.hull);
         let mut shooter_img = spaceship.shooter.image(size)?;
-        shooter_img = rotate90(&shooter_img);
+        shooter_img = if is_player {
+            rotate90(&shooter_img)
+        } else {
+            rotate270(&shooter_img)
+        };
 
         let y_offset = shooter_img.height().saturating_sub(gif[0].height()) / 2;
         let mut shooter_positions = vec![];
@@ -651,16 +787,22 @@ impl SpaceshipEntity {
 
         let used_storage_capacity = resources.used_storage_capacity();
 
-        let shooter = Shooter::new(
-            shooter_positions,
-            spaceship.damage() * weapons_bonus,
-            spaceship.fire_rate(),
-        );
+        let shooter = if spaceship.shooter.fire_rate() == 0.0 {
+            None
+        } else {
+            Some(ShooterInSpaceAdventure::new(
+                shooter_positions,
+                spaceship.damage() * weapons_bonus,
+                spaceship.fire_rate(),
+            ))
+        };
 
-        Ok(Self {
+        let charge_unit = ChargeUnitInSpaceAdventure::new(spaceship.charge_unit.max_charge());
+
+        Ok(Entity::Spaceship(Self {
             id: 0,
-            is_player: true,
-            base_spaceship: spaceship.clone(),
+            is_player,
+            spaceship: spaceship.clone(),
             resources,
             used_storage_capacity,
             storage_capacity: spaceship.storage_capacity(),
@@ -669,7 +811,8 @@ impl SpaceshipEntity {
             gif,
             hit_boxes,
             current_durability: spaceship.current_durability() as f32,
-            durability: spaceship.durability() as f32,
+            max_durability: spaceship.max_durability() as f32,
+            damage_reduction: 1.0,
             base_thrust: spaceship.speed(0) * THRUST_MOD * speed_bonus.powf(0.35), // Assigning the full speed bonus would make the ship too fast
             base_speed: spaceship.speed(0) * MAX_SPACESHIP_SPEED_MOD,
             maneuverability: 0.0,
@@ -679,113 +822,54 @@ impl SpaceshipEntity {
             friction_coeff: FRICTION_COEFF,
             engine_exhaust,
             shooter,
-            auto_shoot: false,
+            charge_unit,
             velocity: Vec2::default(),
             acceleration: Vec2::default(),
             tick: 0,
             collector_id,
+            shield_id,
             visual_effects: HashMap::new(),
-            releasing_scraps: true,
-        })
+            releasing_scraps: false,
+            pending_input_callbacks: Vec::new(),
+        }))
     }
 
-    pub fn random_enemy(collector_id: usize) -> AppResult<Self> {
-        let rng = &mut ChaCha8Rng::from_os_rng();
-        let mut gif = vec![];
-        let mut hit_boxes = vec![];
-        let spaceship = SpaceshipPrefab::iter()
-            .collect_vec()
-            .choose(&mut rand::rng())
-            .expect("There shiuld be one spaceship available")
-            .spaceship("Baddy".to_string())
-            .with_color_map(ColorMap::random(rng));
-
-        let base_gif = spaceship.compose_image()?;
-        for idx in 0..base_gif.len() {
-            let base_image = rotate270(&base_gif[idx]);
-            let (image, hit_box) = body_data_from_image(&base_image, true);
-
-            gif.push(image);
-            hit_boxes.push(hit_box);
-        }
-
-        let position = Vec2::new(SCREEN_SIZE.x as f32, SCREEN_SIZE.y as f32 / 2.0);
-
-        let mut engine_img = open_image(&spaceship.engine.select_mask_file())?;
-        engine_img = rotate270(&engine_img);
-
-        let y_offset = (engine_img.height() - gif[0].height()) / 2;
-        let mut engine_exhaust = vec![];
-        for x in 0..engine_img.width() {
-            for y in 0..engine_img.height() {
-                if let Some(pixel) = engine_img.get_pixel_checked(x, y) {
-                    // If pixel is blue, it is at the exhaust position.
-                    if pixel[0] == 0 && pixel[1] == 0 && pixel[2] == 255 && pixel[3] > 0 {
-                        engine_exhaust.push(I16Vec2::new(x as i16, y as i16 - y_offset as i16));
-                    }
-                }
-            }
-        }
-
-        let size = SpaceshipImage::size(&spaceship.hull);
-        let mut shooter_img = spaceship.shooter.image(size)?;
-        shooter_img = rotate270(&shooter_img);
-
-        let y_offset = (shooter_img.height() - gif[0].height()) / 2;
-        let mut shooter_positions = vec![];
-        for x in 0..shooter_img.width() {
-            for y in 0..shooter_img.height() {
-                if let Some(pixel) = shooter_img.get_pixel_checked(x, y) {
-                    // If pixel is blue, it is at the shooter position.
-                    if pixel[0] == 0 && pixel[1] == 0 && pixel[2] == 255 && pixel[3] > 0 {
-                        shooter_positions.push(I16Vec2::new(x as i16, y as i16 - y_offset as i16));
-                    }
-                }
-            }
-        }
-
-        let resources = ResourceMap::new();
-        let used_storage_capacity = 0;
-        let fuel = spaceship.fuel_capacity() as f32;
-        let storage_capacity = spaceship.storage_capacity();
-        let current_durability = spaceship.current_durability() as f32;
-        let durability = spaceship.durability() as f32;
-        let base_thrust = spaceship.speed(0) * THRUST_MOD;
-        let base_speed = spaceship.speed(0) * MAX_SPACESHIP_SPEED_MOD;
-        let fuel_capacity = spaceship.fuel_capacity();
-        let base_fuel_consumption = spaceship.fuel_consumption_per_tick(0) * FUEL_CONSUMPTION_MOD;
-
-        let shooter = Shooter::new(shooter_positions, spaceship.damage(), spaceship.fire_rate());
-
-        Ok(Self {
-            id: 0,
-            is_player: false,
-            base_spaceship: spaceship,
+    pub fn player_spaceship_entity(
+        spaceship: &Spaceship,
+        resources: ResourceMap,
+        speed_bonus: f32,
+        weapons_bonus: f32,
+        fuel: u32,
+        collector_id: Option<usize>,
+        shield_id: Option<usize>,
+    ) -> AppResult<Entity> {
+        Self::from_spaceship(
+            spaceship,
             resources,
-            used_storage_capacity,
-            storage_capacity,
-            previous_position: position,
-            position,
-            gif,
-            hit_boxes,
-            current_durability,
-            durability,
-            base_thrust,
-            base_speed,
-            maneuverability: 0.0,
+            speed_bonus,
+            weapons_bonus,
             fuel,
-            fuel_capacity,
-            base_fuel_consumption,
-            friction_coeff: FRICTION_COEFF,
-            engine_exhaust,
-            shooter,
-            auto_shoot: false,
-            velocity: Vec2::default(),
-            acceleration: Vec2::default(),
-            tick: 0,
             collector_id,
-            visual_effects: HashMap::new(),
-            releasing_scraps: true,
-        })
+            shield_id,
+            true,
+        )
+    }
+
+    pub fn random_enemy_spaceship_entity(
+        spaceship: &Spaceship,
+        shield_id: Option<usize>,
+    ) -> AppResult<Entity> {
+        let resources = ResourceMap::new();
+
+        Self::from_spaceship(
+            spaceship,
+            resources,
+            1.0,
+            1.0,
+            spaceship.fuel_capacity(),
+            None,
+            shield_id,
+            false,
+        )
     }
 }
