@@ -1,4 +1,3 @@
-// mod asteroids_panel;
 mod big_numbers;
 mod button;
 mod clickable_list;
@@ -13,6 +12,7 @@ mod my_team_panel;
 mod new_team_screen;
 mod player_panel;
 pub mod popup_message;
+mod space_cove_panel;
 mod space_screen;
 mod splash_screen;
 mod swarm_panel;
@@ -21,7 +21,9 @@ pub(crate) mod traits;
 pub mod ui;
 pub mod ui_callback;
 mod ui_frame;
+pub mod ui_key;
 pub mod utils;
 pub mod widgets;
 
 pub use constants::UI_SCREEN_SIZE;
+pub use ui_key::*;
