@@ -1,3 +1,4 @@
+pub mod asteroid_upgrades;
 pub mod constants;
 pub mod honours;
 pub mod jersey;
@@ -8,12 +9,15 @@ pub mod position;
 pub mod resources;
 pub mod role;
 pub mod skill;
+pub mod space_cove;
 pub mod spaceship;
+pub mod spaceship_upgrades;
 pub mod team;
 pub mod types;
 pub mod utils;
 pub mod world;
 
+pub use asteroid_upgrades::*;
 pub use constants::*;
 pub use honours::*;
 pub use jersey::*;
@@ -24,7 +28,9 @@ pub use position::*;
 pub use resources::*;
 pub use role::*;
 pub use skill::*;
+pub use space_cove::*;
 pub use spaceship::*;
+pub use spaceship_upgrades::*;
 pub use team::*;
 pub use types::*;
 pub use utils::*;
